################# PART TO PARAMETRIZE #########################
TEMPLATE	= app
# icpc and g++ are the currently supported parameters, for
COMP            = icpc
#COMP            = g++
# set your processor opteron, pentium-m, pentium4, prescott,
# athlon-xp, athlon64, (for coreDuo use prescott)
# core2Duo
#ARCH            = opteron
#ARCH            = pentium-m
#ARCH            = athlon-xp
ARCH            = core2Duo
#ARCH            = nocona

# activate randomness
RANDOM          = on
# activate debug mode or not
DEBUG           = off
# activate profiling information
PROFIL          = off
# verbose or not
VERBOSE         = off
PARA            = on
# MKL or LAPACK
#LIBALGEBRA = LAPACK
LIBALGEBRA = MKL
SVD = power

# set your path to MKL if you wish to use this library
MKLPATH = /opt/intel/composerxe-2011/mkl/lib/intel64/
MKLPATH_INC = /opt/intel/composerxe-2011/mkl/include/

# if you use the intel compiler, set the path to the libraries 
ICCPATH = /opt/intel/composerxe-2011/
COMP_PATH = /opt/intel/composerxe-2011/bin/


# set the link flags to mkl/blas/lapack depending on which library
# you want to use

equals(LIBALGEBRA,MKL) {
# example for MKL 32 bits
#LIBS *= -lmkl_lapack32 -lmkl_ia32 -lguide -lpthread
# example for MKL 64 bits
LIBS *= -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -liomp5 -lpthread
}

############ END OF PARAMETRIC PART ###########################

equals(LIBALGEBRA,LAPACK) {
# use LAPACL/BLAS insteak
DEFINES *= USE_LAPACK
#LIBS *= -llapack -lblas -lg2c -lm
LIBS *= -lblas -lm
}

equals(SVD,power) {
DEFINES *= SVDPOWER
}


DEFINES += FILENAME_PARAMETER_C=$${PWD}/misc/parameterC

equals(COMP,g++) {
equals(ARCH,core2Duo) {
ARCH = prescott
}
}
LIBS *= -L$${MKLPATH}
system(echo '$$LITERAL_HASH!/bin/sh' > KSVDRestore)
system(echo 'export OMP_NUM_THREADS=4' >> KSVDRestore)
system(echo 'export LD_LIBRARY_PATH=LD_LIBRARY_PATH:$${MKLPATH}:$${ICCPATH}/lib/intel64/' >> KSVDRestore)
system(echo '$${PWD}/bin/KSVDR $*' >> KSVDRestore)
system(chmod +x KSVDRestore)

CONFIG		+= qt release warn_off

DESTDIR = ./bin/

equals(VERBOSE,on) {
DEFINES += KSVD_VERBOSE
DEFINES += CPPL_DEBUG 
}

equals(RANDOM,off) {
DEFINES += NRANDOM
}

equals(LIBALGEBRA,MKL) {
INCLUDE_DIR += $${MKLPATH_INC}
}

# g++
equals(DEBUG,off) {
DEFINES *= NDEBUG
equals(COMP,g++) {
QMAKE_CFLAGS_RELEASE = -O3 -fomit-frame-pointer -mtune=$${ARCH} -pipe 
QMAKE_CXXFLAGS_RELEASE = -O3 -fomit-frame-pointer -mtune=$${ARCH} -pipe 
equals(PARA,on) {
QMAKE_CFLAGS_RELEASE += -fopenmp
QMAKE_CXXFLAGS_RELEASE += -fopenmp
LIBS *= -lgomp
}
}
equals(COMP,icpc) {
LIBS *= -L$${ICCPATH}
INCLUDE_DIR += $${ICCPATH}/include
INCLUDEPATH *= $${ICCPATH}/include/
LIBS *= -L$${ICCPATH}/lib
QMAKE_CFLAGS_RELEASE =  -w0 -DNDEBUG -cxxlib-gcc -ipo -O3 -fp -no-prec-div -fno-alias
QMAKE_CXXFLAGS_RELEASE = -w0 -DNDEBUG -cxxlib-gcc -ipo -O3 -fp -no-prec-div -fno-alias
equals(PARA,on) {
QMAKE_CFLAGS_RELEASE += -openmp -parallel 
QMAKE_CXXFLAGS_RELEASE += -openmp -parallel 
QMAKE_LFLAGS_RELEASE += 
}
# -ipo for max perf, but low compilation
equals(ARCH,pentium-m) {
QMAKE_CFLAGS_RELEASE += -xB 
QMAKE_CXXFLAGS_RELEASE += -xB 
}
equals(ARCH,pentium4) {
QMAKE_CFLAGS_RELEASE += -xN 
QMAKE_CXXFLAGS_RELEASE += -xN 
}
equals(ARCH,prescott) {
QMAKE_CFLAGS_RELEASE += -axP 
QMAKE_CXXFLAGS_RELEASE += -axP 
}
equals(ARCH,core2Duo) {
QMAKE_CFLAGS_RELEASE += -axT 
QMAKE_CXXFLAGS_RELEASE += -axT 
}
#LIBS *= -lunwind
}
}

equals(PROFIL,on) {
QMAKE_CFLAGS_RELEASE += -pg -g 
QMAKE_CXXFLAGS_RELEASE += -pg -g 
QMAKE_LFLAGS_RELEASE += -pg
QMAKE_CFLAGS_RELEASE -= -fomit-frame-pointer
QMAKE_CXXFLAGS_RELEASE -= -fomit-frame-pointer
}

equals(DEBUG,on) {
CONFIG		-= release warn_off
CONFIG		+= debug warn_off
equals(COMP,g++) {
QMAKE_CFLAGS_DEBUG += -ggdb -fno-inline
QMAKE_CXXFLAGS_DEBUG += -ggdb -fno-inline
}
equals(COMP,icpc) {
INCLUDE_DIR += $${ICCPATH}/include
INCLUDEPATH *= $${ICCPATH}/include/
QMAKE_CFLAGS_DEBUG = -O3 -fp -w0 -static -no-prec-div -openmp -cxxlib-gcc -parallel -debug all
QMAKE_CXXFLAGS_DEBUG = -O3 -fp -w0 -static -no-prec-div -openmp -cxxlib-gcc -parallel -debug all
#LIBS *= -lunwind
}
}

MOC_DIR = .moc
OBJECTS_DIR = .obj

PREFIX=/usr
INCLUDE_DIR += $${PREFIX}/include
INCLUDE_DIR += $${PREFIX}/local/include
LIB_DIR += $${PREFIX}/lib
LIB_DIR_64 += $${PREFIX}/lib64
INCLUDEPATH *= $${INCLUDE_DIR}
DEPENDPATH  *= $${INCLUDE_DIR}

LIBS *= -L$${LIB_DIR} -L$${LIB_DIR_64}

TARGET = KSVDR
QMAKE_CC = $${COMP}
QMAKE_CXX = $${COMP_PATH}/$${COMP}
QMAKE_LINK = $${COMP_PATH}/$${COMP}

equals(LIBALGEBRA,MKL) {
# example for MKL 32 bits
#LIBS *= -lmkl_lapack32 -lmkl_ia32 -lguide -lpthread
# example for MKL 64 bits
LIBS *= -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -liomp5 -lpthread
}

HEADERS =         timer.h \
                  image.h \
                  algo.h \
                  processor.h \
                  dictionary.h \
                  inpainting.h \
                  trainer.h \
                  denoising.h \
                  demosaicing.h \
                  video.h \
                  misc.h \
                  svdPower/svdPower.h \
                  #svdlibc/svdlib.h

equals(SVD,power) {
SOURCES = main.cpp 
}
equals(SVD,svdlib) {
SOURCES = main.cpp \
#          svdlibc/svdlib.cc \
}
